#include<io16f877a.h>		//include header file for the PIC
#include"LCDdrive.h"	        //include header file for LCD driver


#define timer_val 75            //this value is set for checking idle time. it means if there is no response for 3 sec after
                                //pressing a buton, system automatically save current character

unsigned char input_array[16] = "000000000000000"; //String for ADC value storage
unsigned char input_array_index = 0; //index of storing array

void read_ch(void); //function for reading character
unsigned char lcd_x =0, lcd_y = 1; //lcd cursor position movement variable
unsigned char pd0=0,pd1=0,pd2=0,pd3=0,pd4=0,pd5=0; //variable corresponding to the input pin
unsigned char press_flag=0,key_pressed=0; //flags forkey press detection and which key is pressed

volatile unsigned char count_TMR1 = 0; //variable for timers' interrupt counter.
unsigned char pin_info = 10; //variable for timers' interrupt counter.

void LCD_msg (void);			//function to show starting message in LCD
void delay_fn (unsigned int delay_val);     //function for software delay
void init_TIMER0(void); //function for timer0 initialization
void deinit_TIMER0(void);//function for timer0 deinitialization

void send_data(void); //function for sending data serilly 
void show_character(unsigned char ch); //function for  display charcter
void onesec_delay (unsigned int y);////function for one sec delay


#pragma vector = 0x04			//interrupt initialisation
__interrupt void sub_routine_ISR (void);	        //interrupt function

void main(void)
{
        RP0=1;                        //set memory block
        PORTD = 0xFF;                //Set all PORT value of D to zero
        TRISD = 0x7F;                //  data direction setting of PORTD

 
        LCD_initialise();		//initialises the LCD 
        LCD_cursor(0,0);            // to choose which line
        LCD_msg();	                // welcome message
        LCD_cursor(0,1);            // setting LCD cursor  
        
        init_TIMER0();

        while(1)                    //infinite loop start
        {
            read_ch();             //all task in one function     
            LCD_cursor_on();        //LCD cursor on
            delay_fn(1000);       //delay for blinking
            
            LCD_cursor_off();        //LCD cursor off
            delay_fn(1000);       //delay for blinking
            
        }
 
}

void LCD_msg (void)
{
        LCD_puts("Welcome       ");	//prints welcome message
        delay_fn(10000); //delay for viewing
        LCD_clear();//clear LCD
        LCD_cursor(0,0); //set LCD cursor to 0,0
        LCD_puts("Text Editor "); //prints current  action
} 

void delay_fn (unsigned int delay_val)    // Functon to implement a short delay
{
	while(delay_val--); //count down of given value
	return;
}

void init_TIMER0(void)
{
     
    
        TMR0 = 0x00; //timer0 initial value is set to aero
        OPTION = 0x06; //prescaler 1/256
        TMR0IE = 1; //timer0 overfow interrupt enable
        GIE = 1;//global interrupt enable
        
}
void deinit_TIMER0(void)
{
        TMR0 = 0x00; //timer0 initial value is set to aero
        OPTION = 0x00; //no clock 
        TMR0IE = 0; //timer0 overfow interrupt disable
        GIE = 0;//global interrupt disable   
}
void read_ch(void)
{
  if(PORTD & 0x7F) //check whether PD6 or PD0 is pressed or not
  {
    unsigned char port_val=0x00; //a variable port_val to generate case value
    port_val = PORTD; //get the PORTD value
    port_val &= 0x7F; //masking the PORTD value
    delay_fn(9000); //delay to suppress debouncing noise from the switch
    switch(port_val) //switch statement
    {
      case 0x00: //nothing is ressed
        break;
      case 0x01: //PD0 is pressed
        LCD_cursor(0,0); //set LCD cursor to 0,0
        LCD_puts("Input character");	//prints "input character"
        LCD_cursor(lcd_x,1); //set LCD cursor to lcd_x,0
        count_TMR1 = timer_val; //for 3 sec
        
        if(pin_info != 0) pd0 = 0; //check whether previous charater is from other pin
        pin_info = 0;
        LCD_putch(97+pd0%6); //display character
        LCD_cursor(lcd_x,1); ////set LCD cursor to lcd_x,0
        pd0++; //update character 
        key_pressed = 0; //which key is presses
        press_flag = 1; //glag for key pressed
        break;
      case 0x02: //PD1 is pressed
        LCD_cursor(0,0);
        LCD_puts("Input character");	//prints "input character"
        LCD_cursor(lcd_x,1);
        count_TMR1 = timer_val; //for 3 sec
        
        if(pin_info != 1) pd1 = 0; //check whether previous charater is from other pin
        pin_info = 1;
        
        LCD_putch(103+pd1%6);
        LCD_cursor(lcd_x,1);
        pd1++;
        key_pressed = 1;
        press_flag = 1;
        break;
      case 0x04: //PD2 is pressed
        LCD_cursor(0,0);
        LCD_puts("Input character");	//prints "input character"
        LCD_cursor(lcd_x,1);
        count_TMR1 = timer_val; //for 3 sec
        
        if(pin_info != 2) pd2 = 0; //check whether previous charater is from other pin
        pin_info = 2;
        
        LCD_putch(109+pd2%7);
        LCD_cursor(lcd_x,1);
        pd2++;
        key_pressed = 2;
        press_flag = 1;
        break;
      case 0x08: //PD3 is pressed
        LCD_cursor(0,0);
        LCD_puts("Input char");	//prints "input character"
        LCD_cursor(lcd_x,1);
        
        if(pin_info != 3) pd3 =0; //check whether previous charater is from other pin
        pin_info = 3;//if the previous character is from the same pin it will go gradually 
        //otherwise from the beginning
        
        count_TMR1 = timer_val; //for 3 sec
        LCD_putch(116+pd3%7);
        LCD_cursor(lcd_x,1);
        pd3++;
        key_pressed = 3;
        press_flag = 1;
        break;
        
      case 0x10: //PD4 is pressed
        LCD_cursor(0,0);
        LCD_puts("Delete char");	//prints "Delete character"
        //lcd_x--;
        LCD_cursor(lcd_x,1);
        count_TMR1 = timer_val; //for 3 sec
        key_pressed = 4;
        press_flag = 1;
        break; 
      case 0x20: //PD5 is pressed
        LCD_cursor(0,0);
        LCD_puts("Space char");	//prints "Space character"
        LCD_cursor(lcd_x,1);
        count_TMR1 = timer_val; //for 3 sec
        LCD_putch(' ');
        LCD_cursor(lcd_x,1);
        key_pressed = 5;
        press_flag = 1;
        break; 
      case 0x40: //PD6 is pressed
        LCD_cursor(0,0);
        
        LCD_puts("Sending.. char:");	//prints "input character"
        key_pressed = 6;
        press_flag = 1;
        break;
    }
  }
  if((count_TMR1 == 0) && press_flag)
  {
    switch(key_pressed)
    {
      case 0:
        input_array[input_array_index] = 97+(pd0-1)%6; //store the pressed value
        input_array_index++; //update the index of storing array
        press_flag = 0; //clear key pressed flag
        lcd_x++; //lcdx_x++ increment the lcd cursor position
        LCD_cursor(lcd_x,1); //set cursor at lcd_x,1
        pin_info = 10; //set the pin track variable value to other value
        break;
      case 1:
        input_array[input_array_index] = 103+(pd1-1)%6;
        input_array_index++;
        press_flag = 0;
        lcd_x++;
        LCD_cursor(lcd_x,1);
        pin_info = 10; //set the pin track variable value to other value
        break;
      case 2:
        input_array[input_array_index] = 109+(pd2-1)%7;
        input_array_index++;
        press_flag = 0;
        lcd_x++;
        LCD_cursor(lcd_x,1);
        pin_info = 10; //set the pin track variable value to other value
        break;
      case 3:
        input_array[input_array_index] = 116+(pd3-1)%7;
        input_array_index++;
        press_flag = 0;
        lcd_x++;
        LCD_cursor(lcd_x,1);
        pin_info = 10; //set the pin track variable value to other value
        break;
      case 4:
        input_array[input_array_index] = '\0'; //put null character for deletion
        input_array_index--; //decrement the index value
        press_flag = 0;
        lcd_x--;
        LCD_cursor(lcd_x,1);
        LCD_putch(' ');
        LCD_cursor(lcd_x+1,1);
        LCD_cursor(lcd_x,1);
        break;    
      case 5:
        input_array[input_array_index] = 32; //space character value
        input_array_index++;
        press_flag = 0;
        lcd_x++;
        LCD_cursor(lcd_x,1);
        break;
      case 6:
        press_flag = 0;
        lcd_x = 0;
        deinit_TIMER0(); //disable interrupt based timer for sending data
        input_array[input_array_index] = '\0'; //store the pressed value
        
        send_data(); //data send
        input_array_index =0; //clear array index
        init_TIMER0(); //enable inerrupt based timer for taking input
        break;  
      default:
        press_flag = 0; //clear key pressed flag
        break;  
    }
  }
}

void send_data(void)
{
  unsigned char i = 0; //local variable
  for(i=0;i<=input_array_index;i++) //send value upto the stored 
  {
    show_character(input_array[i]);//send one charcter
    LCD_cursor(15,0); //LCD cursor at 15,0
    LCD_putch(input_array[i]); //show the sending character
    delay_fn(9000);//delay 
  }
}
void show_character(unsigned char ch)
{
  unsigned char i=0;
  PORTD = 0x00; //start bit
  onesec_delay(24); //one sec delay to visible the led blinkng
  for(i=0;i<8;i++) //loop for 8 bit
  {
    if(ch & 1) //if the current bit is one
    {
      PORTD = 0x80; //set one
      onesec_delay(24);//one sec delay,baud ate is 1 to see the LED blinking
    }
    else
    {
      PORTD = 0x00; //set zero
      onesec_delay(24); //one sec delay
    }
    ch>>=1; //next bit
  }
    PORTD = 0x80; //stop bit
    onesec_delay(24); //one sec delay to visible the led blinkng
}

void onesec_delay (unsigned int y)	// one second delay fuction y =24 for 1sec,12 for 500m, 6 for 250m
{
  unsigned char k;
  for(k=0;k<y;k++)
	{
		TMR0=0;		        //start TMR0
		while(TMR0-133);	//TMR0 - 133
         } 
         return;	//return to the calling function
}
__interrupt void sub_routine_ISR (void)
{
        GIE = 0;  //global interrupt disable

        if(TMR0IF)//checking for timer0 interrupt
        {
                
                TMR0IF = 0; //clear timer0 overflow interrupt
                
                if(count_TMR1) //if count_TMR1 value is not zero
                {
                  count_TMR1--; //decrement the couter value for exact idle time checking
                }
        }

        GIE = 1; //global interrupt enable
}
